@@ -1,6 +1,5 @@
-from math import log
 from operator import and_
-from struct import calcsize, unpack
+from struct import calcsize, unpack, iter_unpack
 from warnings import warn
 import os
 import re
@@ -173,14 +172,8 @@ class FlowData(object):
             # from here on out we assume mode "l" (list)
 
         bit_width = []
-        data_range = []
         for i in range(1, int(text['par']) + 1):
             bit_width.append(int(text['p%db' % i]))
-            try:
-                data_range.append(int(text['p%dr' % i]))
-            except ValueError:
-                # Found an FCS channel using exponential notation for the int
-                data_range.append(int(float(text['p%dr' % i])))
 
         if data_type.lower() == 'i':
             data = self.__parse_int_data(
@@ -188,7 +181,6 @@ class FlowData(object):
                 start,
                 stop,
                 bit_width,
-                data_range,
                 order
             )
         else:
@@ -202,7 +194,7 @@ class FlowData(object):
 
         return data
 
-    def __parse_int_data(self, offset, start, stop, bit_width, d_range, order):
+    def __parse_int_data(self, offset, start, stop, bit_width, order):
         """Parse out and return integer list data from FCS file"""
 
         if reduce(and_, [item in [8, 16, 32] for item in bit_width]):
@@ -241,35 +233,24 @@ class FlowData(object):
                 )
 
             # parameter sizes are different
-            # e.g. 8, 8, 16,8, 32 ... do one at a time
+            # e.g. 8, 8, 16, 8, 32 ...
             else:
-                log2 = self.__log_factory(2)
-                unused_bit_widths = list(map(int, map(log2, d_range)))
-                tmp = []
-                cur = start
-                while cur < stop:
-                    for i, cur_width in enumerate(bit_width):
-                        bit_mask = self.__mask_integer(
-                            cur_width,
-                            unused_bit_widths[i])
-                        n_bytes = cur_width / 8
-                        bin_string = self.__read_bytes(
-                            offset, cur,
-                            cur + n_bytes - 1)
-                        cur += n_bytes
-                        val = bit_mask & unpack(
-                            '%s%s' %
-                            (
-                                order,
-                                self.__format_integer(cur_width)
-                            ),
-                            bin_string)[0]
-                        tmp.append(val)
+                tmp = self.__extract_var_length_int(bit_width, offset, order, start, stop)
+
         else:  # non standard bit width...  Does this happen?
             warn('Non-standard bit width for data segments')
             return None
         return tmp
 
+    def __extract_var_length_int(self, bit_width, offset, order, start, stop):
+        data_format = order
+        for cur_width in bit_width:
+            data_format += '%s' % self.__format_integer(cur_width)
+
+        tuple_tmp = iter_unpack(data_format, self.__read_bytes(offset, start, stop))
+        tmp = [ti for t in tuple_tmp for ti in t]
+        return tmp
+
     def __parse_non_int_data(self, offset, start, stop, data_type, order):
         """Parse out and return float or ASCII list data from FCS file"""
         data_type_size = calcsize(data_type)
@@ -332,29 +313,9 @@ class FlowData(object):
         elif b == 32:
             return 'I'
         else:
-            print("Cannot handle integers of bit size %d" % b)
-            return None
-
-    @staticmethod
-    def __mask_integer(b, ub):
-        """return bit mask of an integer and a bit width"""
-        if b == 8:
-            return 0xFF >> (b - ub)
-        elif b == 16:
-            return 0xFFFF >> (b - ub)
-        elif b == 32:
-            return 0xFFFFFFFF >> (b - ub)
-        else:
-            print("Cannot handle integers of bit size %d" % b)
-            return None
-
-    @staticmethod
-    def __log_factory(base):
-        """factory for various bases or the log function"""
-        def f(x):
-            return log(x, base)
-
-        return f
+            raise ValueError(
+                "Invalid integer bit size (%d) for event data. Compatible sizes are 8, 16, & 32." % b
+            )
 
     def _parse_channels(self):
         """
